@@ -15,6 +15,23 @@ use RapidApp::Module::AppDV::TTController;
 
 use HTML::TokeParser::Simple;
 
+# If true, the template will refresh itself (client-side) after saving to the store.
+# This will make the whole template refresh instead of just the updated rows. This
+# is useful when updates involve changes to more than one row/data-point on the backend
+has 'refresh_on_save', is => 'ro', isa => 'Bool', default => 0, traits => ['ExtProp'];
+
+# Only makes sense when there is exactly one row; will beging editing the *first* record
+# as soon as the page/data is loaded. It is the developer's responsibility to 
+# include [% r.toggle.edit %] someplace in the template so the user can actually
+# save the changes. If the toggle control is not included, or it is otherwise not
+# possible for the user to have manually started the edit, this setting will do nothing.
+has 'init_record_editable', is => 'ro', isa => 'Bool', default => 0, traits => ['ExtProp'];
+
+# If true, the template will refresh itself (client-side) when the window.location.hash
+# changes. Within templates, the value of the hash (without '#') is available as:
+#   {[this.hashval]}
+has 'refresh_on_hash_change', is => 'ro', isa => 'Bool', default => 0, traits => ['ExtProp'];
+
 # If true, the content will be parsed for a <title> tag to set on the tab in
 # the same manner as this works on the JavaScript side for plain html content
 has 'parse_html_tabTitle', is => 'ro', isa => 'Bool', default => 1;
@@ -184,8 +201,14 @@ sub xtemplate_funcs {
     disableFormats => \1,
     renderField => RapidApp::JSONFunc->new( raw => 1, func =>
       'function(name,values,renderer) {' .
-        'var record = { data: values };' .
-        'return renderer(values[name],{},record);' .
+        #'var record = { data: values };' .
+        #'return renderer(values[name],{},record);' .
+        'var dsp = this.store.datastore_plus_plugin;'.
+        'var record = this.store.getById(values.___record_pk);' .
+        'var args = [values[name],{},record];' .
+        'return dsp._masterColumnRender(' .
+          '{ renderer: renderer, args: args, name: name }'.
+        ');'.
       '}'
     )
   };
