@@ -16,8 +16,18 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlusX = {
     if(ed) {
       var field = ed.field;
       if(field && !field.DataStorePlusApplied) {
+      
+        field.reportDirtyDisplayVal = function(disp) {
+          var Record = cmp.bindRecord || ed.record; // bindRecord applies to PropertyGrid
+          Record._dirty_display_data = Record._dirty_display_data || {};
+          Record._dirty_display_data[field.name] = disp;
+        }
         
-        var stopEditFn = cmp.stopEditing.createDelegate(cmp);
+        var _stopEditFn = cmp.stopEditing.createDelegate(cmp);
+        var stopEditFn = function() {
+          _stopEditFn();
+          cmp.view.refresh();
+        }
         
         // --- Handle Ctrl+S/Ctrl+Z ('save'/'undo' keyboard shortcuts) for in-progress edit:
         field.on('afterrender', function(){
@@ -77,6 +87,7 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 	init: function(cmp) {
 		this.cmp = cmp;
 		cmp.datastore_plus_plugin = this;
+		cmp.store.datastore_plus_plugin = this; // also save a ref in the store
     
     // ---
     // Auto-cleanup of entries in Ext.StoreMgr on destroy
@@ -207,6 +218,8 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
     else {
       this.cmp.on('render',this.initializeStoreButtons,this);
     }
+    
+    this.cmp.on('render',this.updateCssForStoreAPI,this);
 		
 		// Only applies to editor grids; no effect/impact on other components
 		// without the beforeedit/afteredit events
@@ -307,6 +320,13 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 		}
     
     cmp.addEvents('firstload');
+    cmp.onFirstLoad = function(ds){
+      ds.un('load',cmp.onFirstLoad);
+      // args contains our custom first two elements following by our arguments
+      var args = ['firstload',cmp].concat([].slice.call(arguments));
+      cmp.fireEvent.apply(cmp,args);
+    }
+    cmp.store.on('load',cmp.onFirstLoad,cmp.store);
     
     cmp.on('render',function() {
       var El = Ext.isFunction(cmp.getLoadMaskEl) // <-- Allow the cmp to define (AppDV)
@@ -347,7 +367,7 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
           lMask.show();
           var hide_fn;
           hide_fn = function(){ 
-            cmp.fireEvent('firstload');
+            //cmp.fireEvent('firstload');
             lMask.hide(); 
             cmp.store.un('load',hide_fn);
             cmp.store.un('exception',hide_fn); 
@@ -577,6 +597,9 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 				// request to request, so we can disable the edit button when that number is 0
 				store.editable_fields_count = edit_count;
 			}
+      
+      this.updateCssForStoreAPI();
+      
 		},this);
 		
 		store.hasLoadedColumn = function(name) {
@@ -742,7 +765,9 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 					height: height,
 					closable: true,
 					modal: true,
-					items: formpanel
+					items: formpanel,
+          border: false,
+          smartRenderTo: plugin.cmp
 				},plugin.add_form_window_cnf);
 				
 				if(Ext.isFunction(plugin.cmp.add_form_onPrepare)) {
@@ -882,7 +907,9 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 					height: height,
 					closable: true,
 					modal: true,
-					items: formpanel
+					items: formpanel,
+          border: false,
+          smartRenderTo: plugin.cmp
 				},plugin.add_form_window_cnf); //<-- use same custom config from add
 				
 				if(Ext.isFunction(plugin.cmp.edit_form_onPrepare)) {
@@ -1140,6 +1167,39 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
         store.undoChanges.call(store);
       }
     },store);
+    
+    
+    // ------
+    //  NEW: track the last response AND decoded data in a common location for both read/write:
+    //
+    // Here we're hooking into the store events to capture the already decoded json data
+    store.lastJsonData = null;
+    store.on('load',function(store) {
+      store.lastJsonData = store.reader.jsonData;
+    },this);
+    store.on('write',function(store,action,result,res) {
+      if(res && res.raw) {
+        store.lastJsonData = res.raw;
+      }
+    },this);
+    
+    // Here we're reaching further down into the stack to capture the real Ajax response object
+    // which is not available by hooking the store 'load' event, as we're doing above. The reason
+    // we are doing both of these is to make all of the data available w/o having to decode the
+    // responseText twice. Since this is deeper, lastResponse is available slighter sooner than 
+    // lastJsonData. lastJsonData (above) should be the same as: Ext.decode(lastResponse.responsetext)
+    if(store.proxy && store.proxy instanceof Ext.data.HttpProxy) {
+      store.proxy.lastResponse = null;
+      store.proxy.onRead = function(action, o, response) {
+        this.lastResponse = response;
+        return this.__proto__.onRead.apply(this,arguments);
+      };
+      store.proxy.onWrite = function(action, o, response) {
+        this.lastResponse = response;
+        return this.__proto__.onWrite.apply(this,arguments);
+      };
+    }
+    // ------
 	},
 	
 	// Only applies to Editor Grids implementing the 'beforeedit' event
@@ -1550,6 +1610,28 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 		Ext.each(tbar_items,function(btn) { tbar.insert(0,btn); },this);
 		
 	},
+  
+  // Sets/clears a CSS class/flag for each CRUD action missing from the store api
+  updateCssForStoreAPI: function() {
+    var El = this.cmp.getEl(), store = this.cmp.store;
+    var apis = ['create','read','update','destroy'];
+    var testFn = function(api) {
+      if(!store.api[api]) { return false; }
+      // Extra test for update -- 
+      // if there are no editable columns, consider update as a whole denied:
+      if(api == 'update' && store.editableFieldsCount() == 0) { return false; }
+      return true;
+    };
+    Ext.each(apis,function(api){
+      var cls = ['ra-dsapi-deny-',api].join('');
+      if(testFn(api)) {
+        if(El.hasClass(cls)) { El.removeClass(cls); }
+      }
+      else {
+        if(!El.hasClass(cls)) { El.addClass(cls); }
+      }
+    },this);
+  },
 	
 	beforeRemoveConfirm: function(c,component) {
 		if(component != this.cmp) {
@@ -1642,11 +1724,18 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 			
 			// Use a copy of the new record in case the save fails and we need to try again:
 			var newRecord = newRec.copy();
+      if(newRec._dirty_display_data) {
+        newRecord._dirty_display_data = newRec._dirty_display_data;
+      }
 			newRecord.phantom = true; //<-- the copy doesn't have this set like the original... why?
 			
 			form.updateRecord(newRecord);
 			
 			store.add(newRecord);
+      
+      if(plugin.cmp.view && plugin.cmp.view.refresh){
+        plugin.cmp.view.refresh();
+      }
 			
 			if(plugin.persist_immediately.create) {
 				
@@ -1699,6 +1788,13 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
         }
       },this);
       
+      Ext.each(formpanel.items,function(field) {
+        field.reportDirtyDisplayVal = function(disp) {
+          newRec._dirty_display_data = newRec._dirty_display_data || {};
+          newRec._dirty_display_data[field.name] = disp;
+        }
+      },this);
+      
       formpanel.Record = newRec;
       
       hide_mask();
@@ -1761,6 +1857,10 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 			store.on('exception',fp_enable_handler,this);
 			
 			form.updateRecord(Rec);
+      
+      if(plugin.cmp.view && plugin.cmp.view.refresh){
+        plugin.cmp.view.refresh();
+      }
 			
 			if(plugin.persist_immediately.update) {
 				
@@ -1830,6 +1930,12 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 					// Don't try to edit fields that aren't loaded, exclude them from the form:
 					if(!store.hasLoadedColumn(field.name)){ return; }
 					field.value = Rec.data[field.name];
+          
+          field.reportDirtyDisplayVal = function(disp) {
+            Rec._dirty_display_data = Rec._dirty_display_data || {};
+            Rec._dirty_display_data[field.name] = disp;
+          }
+          
 					new_items.push(field);
 				},this);
 				formpanel.items = new_items;
@@ -1850,9 +1956,26 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 			},
 			scope: this
 		});
-	}
+	},
 	// --- ^^ ---
 
+  // NEW: rendering for all columns now passes through this common/master, private wrapper function:
+  _masterColumnRender: function(p) {
+  
+    var Rec = p.args[2], dirtyRender = (
+      Rec && Rec.dirty && p.name && Rec.modified && Rec._dirty_display_data 
+      && typeof Rec.modified[p.name] != 'undefined'
+      && typeof Rec._dirty_display_data[p.name] != 'undefined'
+    ) ? true : false;
+    
+    if(dirtyRender) {
+      var disp = Rec._dirty_display_data[p.name];
+      return [ '<span class="ra-dirty-display-val">',Ext.ux.showNull(disp),'</span>'].join('');
+    }
+
+    var scope = p.scope || this;
+    return p.renderer.apply(scope,p.args);
+  }
 });
 Ext.preg('datastore-plus',Ext.ux.RapidApp.Plugin.CmpDataStorePlus);
 
