@@ -408,20 +408,63 @@ Ext.override(Ext.form.Field, {
 
 
 
-// We never want Windows to render larger than the browser. I can't imagine any situation
-// where this would be wanted, so this is being implemented as a global override for now:
 var orig_Window_initComponent = Ext.Window.prototype.initComponent;
 Ext.override(Ext.Window, {
   initComponent: function() {
+    
+    // More flexible way to supply renderTo for a window to contrain
+    if(this.smartRenderTo) {
+      var El = Ext.isFunction(this.smartRenderTo.getEl) 
+        ? this.smartRenderTo.getEl() 
+        : this.smartRenderTo;
+      
+      // ExtJS is full of CSS style bugs when trying to nest things within grid elements. It
+      // breaks scrolling, changes borders, etc, because of improperly overly-broad rules.
+      // So if this is a grid, jump up one element higher to escape this CSS space
+      //  (See GitHub Issue #6 for more info)
+      if(El.hasClass('x-grid-panel')) { El = El.parent(); }
+      
+      // Special handling -- do not constrain/renderTo at all if we're already nested
+      // in an existing window (i.e. fall-back to full-browser. This was added to handle
+      // the case (specific to RapidApp) of add-and-select a related row in which the
+      // related grid is opened in a window which provides an "Add New" button. Regardless,
+      // its doubtful constraining across multiple layers of windows would ever be the
+      // desired behaviour in any case.
+      if(! El.parent('div.x-window-body')) {
+        this.renderTo = El;
+      }
 
+      // Force enable 'constrain' for smartRenderTo -- this ensures that the window stays
+      // within the borders of the target element. Without it, the window could be rendered
+      // off of the screen if an error occurs when calculating alignment (this happens if
+      // the target smartRenderTo is not visible at the time the window is shown.)
+      // TODO: properly figure out and handle this case. For now, turning this on solves
+      // the immediate problem, but the window gets rendered in the top-left instead of the
+      // center, and also has some odd sizing issues.
+      this.constrain = true;
+    }
+
+    // We never want Windows to render larger than the browser. I can't imagine any situation
+    // where this would be wanted, so this is being implemented as a global override for now:
     var browserSize = Ext.getBody().getViewSize();
     var maxW = browserSize.width  - 10;
     var maxH = browserSize.height - 10;
 
+    // If we're rendering to a renderTo Element, limit max size further down to it:
+    if(this.renderTo && this.renderTo instanceof Ext.Element) {
+      var renderToSize = this.renderTo.getViewSize();
+      if(renderToSize && renderToSize.width && renderToSize.height) {
+        var w = renderToSize.width - 10;
+        var h = renderToSize.height - 10;
+        maxW = w < maxW ? w : maxW;
+        maxH = h < maxH ? w : maxH;
+      }
+    }
+
     // For now, only handle the case of supplied/set height & width values:
     if(Ext.isNumber(this.height)) { this.height = this.height < maxH ? this.height : maxH; }
     if(Ext.isNumber(this.width))  { this.width  = this.width  < maxW ? this.width  : maxW; }
-
+    
     return orig_Window_initComponent.call(this);
   }
 });
@@ -462,12 +505,14 @@ Ext.override(Ext.form.DateField, {
         },{ scope: cmb, delay: 50 });
 
         if(this.ownerCt) {
-          var xtype = this.ownerCt.getXType();
-          if(xtype && xtype == 'xdatetime2' && this.ownerCt.ownerCt){
+          var oCt = this.ownerCt;
+          var xtype = oCt.getXType();
+          if(xtype && xtype == 'xdatetime2' && oCt.ownerCt){
             // if we're part of a datetime, consider its ownerCt:
-            xtype = this.ownerCt.ownerCt.getXType();
+            oCt = oCt.ownerCt;
+            xtype = oCt.getXType();
           }
-          if(xtype && xtype == 'form') {
+          if((xtype && xtype == 'form') || oCt instanceof Ext.form.FormPanel) {
             // If we're in a form, we want to start the toggle state on, so that the
             // first click will will trigger instead of the second. The reason is that
             // forms do not trigger the toggle automatically, while other contexts (i.e.
@@ -494,6 +539,17 @@ Ext.override(Ext.form.DateField, {
       input = input.split('T')[0];
     }
     return orig_DateField_parseDate.call(this, input, format);
+  },
+  onMenuHide: function(){
+    // This focus call comes from ExtJS core and seems to be a bug -- why would we want to
+    // focus the field when we hide the menu? This is at best not very helpful, and at worst 
+    // harmful if for example the reason the menu hides is because the user clicked on a 
+    // different field; this steals the focus back. If the user wants to manually type after
+    // starting with the menu, they can click to focus the field themselves (which already
+    // hides the menu -- if the field doesn't have focus, it probably means the user doesn't
+    // want it to).
+    //this.focus(false, 60);
+    this.menuEvents('un');
   }
 });
 
@@ -508,3 +564,122 @@ Ext.form.HtmlEditor.prototype.getDoc = function() {
   return win ? win.document : null;
 }
 
+// NEW: hook 'afterrender' on _every_ component class to call the function
+// to look for and load our special 'ra-async-box' tags (EXPERIMENTAL)
+var orig_Component_initComponent = Ext.Component.prototype.initComponent;
+Ext.override(Ext.Component,{
+  initComponent: function() {
+    orig_Component_initComponent.apply(this,arguments);
+    this.on('afterrender',function() {
+      Ext.ux.RapidApp.loadAsyncBoxes(this);
+    },this,{delay:200});
+  }
+});
+
+// DataView's refresh() doesn't fire afterrender, so we handle it manually
+// (Note: this is not called by AppDV because it implements its own refresh() which
+// does not call the method from its superclass -- AppDV also manually calls this)
+var orig_DataView_refresh = Ext.DataView.prototype.refresh;
+Ext.override(Ext.DataView,{
+  refresh: function() {
+    orig_DataView_refresh.apply(this,arguments);
+    Ext.ux.RapidApp.loadAsyncBoxes.defer(150,this,[this]);
+  }
+});
+
+
+Ext.override(Ext.grid.GridView, {
+
+    // ********************************************************************************** //
+    // ** copy/paste of the entire orioginal doRender() from Ext.grid.GridView (3.4.0) **
+    // While having to resort to doing this is unfortunate,there is just no other way
+    // to extend and move forward with a more sophisticated model for column rendering.
+    // Because of the way the code is structured, there is no cleaner place to hook. 
+    // this is a copy/paste of the original function, changing only the line which 
+    // calls the renderer from the column model/object
+    doRender : function(columns, records, store, startRow, colCount, stripe) {
+        var templates = this.templates,
+            cellTemplate = templates.cell,
+            rowTemplate = templates.row,
+            last = colCount - 1,
+            tstyle = 'width:' + this.getTotalWidth() + ';',
+            // buffers
+            rowBuffer = [],
+            colBuffer = [],
+            rowParams = {tstyle: tstyle},
+            meta = {},
+            len  = records.length,
+            alt,
+            column,
+            record, i, j, rowIndex;
+
+        //build up each row's HTML
+        for (j = 0; j < len; j++) {
+            record    = records[j];
+            colBuffer = [];
+
+            rowIndex = j + startRow;
+
+            //build up each column's HTML
+            for (i = 0; i < colCount; i++) {
+                column = columns[i];
+                
+                meta.id    = column.id;
+                meta.css   = i === 0 ? 'x-grid3-cell-first ' : (i == last ? 'x-grid3-cell-last ' : '');
+                meta.attr  = meta.cellAttr = '';
+                meta.style = column.style;
+
+                // *** modified line(s) *** //
+                var dsp = store.datastore_plus_plugin;
+                if(dsp) {
+                  meta.value = dsp._masterColumnRender({
+                    name: column.name, renderer: column.renderer, scope: column.scope,
+                    args: [record.data[column.name], meta, record, rowIndex, i, store]
+                  });
+                }
+                else {
+                  // original code:
+                  meta.value = column.renderer.call(column.scope, record.data[column.name], meta, record, rowIndex, i, store);
+                }
+                // ************************ //
+
+                if (Ext.isEmpty(meta.value)) {
+                    meta.value = '&#160;';
+                }
+
+                if (this.markDirty && record.dirty && typeof record.modified[column.name] != 'undefined') {
+                    meta.css += ' x-grid3-dirty-cell';
+                }
+
+                colBuffer[colBuffer.length] = cellTemplate.apply(meta);
+            }
+
+            alt = [];
+            //set up row striping and row dirtiness CSS classes
+            if (stripe && ((rowIndex + 1) % 2 === 0)) {
+                alt[0] = 'x-grid3-row-alt';
+            }
+
+            if (record.dirty) {
+                alt[1] = ' x-grid3-dirty-row';
+            }
+
+            rowParams.cols = colCount;
+
+            if (this.getRowClass) {
+                alt[2] = this.getRowClass(record, rowIndex, rowParams, store);
+            }
+
+            rowParams.alt   = alt.join(' ');
+            rowParams.cells = colBuffer.join('');
+
+            rowBuffer[rowBuffer.length] = rowTemplate.apply(rowParams);
+        }
+
+        return rowBuffer.join('');
+    }
+    // ********************************************************************************** //
+
+});
+
+
