@@ -18,6 +18,24 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
 		return this;
 	},
 	
+  // collectData() is called whenever rendering/refreshing the template:
+  collectData: function() {
+    // New: Save the latest response data within the XTemplate object via the
+    // reference keys which are now made available via DataStorePlus. This
+    // enables the template environment to get this data via '[{this.resData}]'
+    try{
+      var hashval = window.location.hash;
+      if(hashval && hashval.search('#') == 0) { 
+        hashval = hashval.substring(1); 
+      }
+      this.tpl.hashval = hashval;
+      this.tpl.resData = this.store.lastJsonData
+        // The lastJsonData might not be populated on the first load, for this 
+        // case,   // reach into the lastResponse, which is also now tracked, and decode it
+        || Ext.decode(this.store.proxy.lastResponse.responseText);
+    }catch(err){};
+    return Ext.ux.RapidApp.AppDV.DataView.superclass.collectData.apply(this,arguments);
+  },
   
   getLoadMaskEl: function() {
     var El = this.getEl();
@@ -40,6 +58,8 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
       if(this.find_clickableEl(event)){ return false; };
     },this);
     
+    this.tpl.store = this.store;
+    
 		//if(!this.store) { this.store = this.ownerCt.store; }
 		
 		this.store.on('beforesave',this.onBeforesave,this);
@@ -70,6 +90,36 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
 		},this);
 		
 		this.on('beforeselect',this.onBeforeselect,this);
+    
+    if(this.refresh_on_save) {
+      this.store.on('write',function(){
+        this.refresh.apply(this,arguments);
+      },this);
+    }
+    
+    if(this.init_record_editable) {
+      this.on('firstload',function(cmp,ds){
+        var node = this.getNode(0,0);
+        if(node) {
+          var El = new Ext.Element(node);
+          var Toggle = El.child('div.edit-record-toggle');
+          if(Toggle) {
+            Toggle.dom.click();
+          }
+        }
+      },this);
+    }
+    
+    if(this.refresh_on_hash_change) {
+      Ext.History.on('change',this.refresh,this);
+      this.on('beforedestroy',function(){
+        Ext.History.un('change',this.refresh,this);
+      },this);
+    }
+    
+    // init
+    this.FieldCmp = {};
+    
 	},
 	
 	onBeforeselect: function() {
@@ -111,7 +161,11 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
     this.hasSkippedEmptyText = true;
     // ---
 
+    // renderItems relates to the special case of sub-componenets, not 
+    // rendering the normal, local records/columns and their editors
     this.renderItems(0, this.store.getCount() - 1);
+    
+    this.toggleDirtyCssRecord(records,false);
 	},
 	
 	onUpdate: function(ds, record){
@@ -257,6 +311,16 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
 				//
 				//rec.dirtyEl = el.insertSibling(domCfg,'before');
 			}
+      
+      if(rec.phantom) {
+        if(el.hasClass('non-phantom')) { el.removeClass('non-phantom'); }
+        if(!el.hasClass('is-phantom')) { el.addClass('is-phantom'); }
+      }
+      else {
+        if(el.hasClass('is-phantom')) { el.removeClass('is-phantom'); }
+        if(!el.hasClass('non-phantom')) { el.addClass('non-phantom'); }
+      }
+      
 		},record);
 	},
 	
@@ -369,6 +433,9 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
 			}
 		}
 		this.components.splice.apply(this.components, args);
+    
+    // re-scan for ra-async-box elements -- codepath is specific to AppDV
+    Ext.ux.RapidApp.loadAsyncBoxes(this);
 	},
 	
 	destroyItems: function(index){
@@ -430,7 +497,9 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
 		var dataWrap = editEl.child('div.data-wrapper');
 		var dataEl = editEl.child('div.data-holder');
 		var fieldEl = editEl.child('div.field-holder');
-		
+    
+    if(!this.fieldRecordCanEdit(fieldname,Record)) { return; }
+
 		
 		editEl.addClass('editing');
 
@@ -449,8 +518,8 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
     }
     
 		
-		if(!cnf.width) {	cnf.width = dataEl.getWidth(); }
-		if(!cnf.height) { cnf.height = dataEl.getHeight(); }
+		if(!cnf.width) {	cnf.width = dataEl.getWidth() + 10; }
+		if(!cnf.height) { cnf.height = dataEl.getHeight() + 6; }
 		if(cnf.minWidth) { if(!cnf.width || cnf.width < cnf.minWidth) { cnf.width = cnf.minWidth; } }
 		if(cnf.minHeight) { if(!cnf.height || cnf.height < cnf.minHeight) { cnf.height = cnf.minHeight; } }
 		
@@ -468,8 +537,23 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
 		//}
 		
 		var Store = this.getStore();
+    
+    cnf.AppDv_context = {
+      appdv: this,
+      Record: Record,
+      editEl:editEl,
+      index: index
+    };
+    cnf.plugins = cnf.plugins || [];
+    cnf.plugins.push('appdv-field-plugin');
+    
 		
 		var Field = Ext.create(cnf,'field');
+    
+    Field.reportDirtyDisplayVal = function(disp) {
+      Record._dirty_display_data = Record._dirty_display_data || {};
+      Record._dirty_display_data[Field.name] = disp;
+    }
 		
 		if(!Ext.isObject(this.FieldCmp)) { this.FieldCmp = {} }
 		if(!Ext.isObject(this.FieldCmp[index])) { this.FieldCmp[index] = {} }
@@ -586,12 +670,14 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
 		var dataEl = editEl.child('div.data-holder');
 		var fieldEl = editEl.child('div.field-holder');
 		
-		if(dataWrap && dataEl && fieldEl) {
-
+    
+    if(this.FieldCmp[index] && this.FieldCmp[index][fieldname] && dataWrap && dataEl && fieldEl) {
 			var Fld = this.FieldCmp[index][fieldname];
 			if(Fld.contentEl) {
 				Fld.contentEl.appendTo(dataWrap);
 			}
+      // remove the field valid state from consideration (will clear if the field was invalid)
+      this.fireEvent('valid',Fld); 
 			Fld.destroy();
 			dataEl.setVisible(true);
 			
@@ -673,26 +759,59 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
     // the ordinary browser event (by calling e.preventDefault()). This will stop
     // ordinary links from working. So, if we want links to work, we have to handle
     // here, manually:
-    
-    var target = event.getTarget(null,null,true);
-    if(target.is('a')) {
-      if(this.allow_all_links) {
-        return false;
-      }
+  
+    // testTarget()
+    // Returns true if the supplied target should be handled by AppDV, false if
+    // it should be handled natively by the browser
+    var testTarget;
+    testTarget = function(target) {
       // We're still not going to allow *ALL* links through... We are letting
       // through links with the special 'filelink' class, and also letting 
       // through links which have defined a target (i.e. target="_blank", etc),
       // otherwise we will continue with the existing native behavior which is
-      // to make the link do nothing. Also, note that we are now enabling the
+      // to make the link do nothing in the browser, but be handled in AppDV. 
+      // Also, note that we are now enabling the
       // 'ra-link-click-catcher' in AppDV per default (see AppDV.pm) so it may
       // still pickup and handle relative URL links in the standard manner.
       if(
         target.hasClass('filelink') ||
-        target.getAttribute('target')
-      ) { return false; }
-    }
+        target.getAttribute('target') || 
+        // New: if this is a form submit button/input, assume the user means it
+        // to submit as usual and allow it through:
+        (target.getAttribute('type') && target.getAttribute('type')  == 'submit')
+      ) {
+        // If we're here it means the target matches one of our exclusions, so we'll
+        // return false to allow the browser to handle it **unless** it is within a
+        // declared, appdv clickable element. We always want to handle these ourselves
+        var click_parent = target.parent('div.clickable');
+        if(click_parent && click_parent.parent('div.appdv-tt-generated')){
+          return true;
+        }
+        else {
+          return false; 
+        }
+      }
+      
+      if(target.is('a')) {
+        if(this.allow_all_links) {
+          return false;
+        }
+      }
+      else {
+        // If we're not an <a> tag, check to see if we are within an <a> tag, and
+        // for that case consider the parent <a> tag:
+        var parent = target.parent('a');
+        if(parent) {
+          return testTarget(parent);
+        }
+      }
+      
+      return true;
+    };
+    
+    return testTarget( event.getTarget(null,null,true) );
   },
-
+  
 	click_controller: function(dv, index, domNode, event) {
     var clickableEl = this.find_clickableEl.call(dv,event,domNode);
     
@@ -764,7 +883,7 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
 		var Store = this.getStore();
 		
 		var fieldname = this.get_fieldname_by_editEl(editEl);
-		if(!fieldname) { return; }
+		if(!this.fieldRecordCanEdit(fieldname,Record)) { return; }
 		
 		var dataWrap = editEl.child('div.data-wrapper');
 		var dataEl = editEl.child('div.data-holder');
@@ -830,6 +949,24 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
 	simulateCancelClick: function(Record,index,editEl) {
 		return this.simulateEditRecordClick('div.cancel',Record,index,editEl);
 	},
+  
+  // Returns true/false if the field can edit, properly considering edit vs add
+  fieldRecordCanEdit: function(fieldname,Record) {
+    if(!fieldname || !Record) { return false; }
+    var column = Record.store.getColumnConfig(fieldname);
+    if(!column) { return false; }
+    if(Record.phantom) {
+      if(!column.allow_add) { return false; }
+      if(!Record.store.api.create) { return false; }
+    }
+    else {
+      if(!column.allow_edit) { return false; }
+      if(!Record.store.api.update) { return false; }
+    }
+    
+    return true;
+  },
+  
 	
 	handle_edit_record: function (target,editEl,Record,index,domEl) {
 		
@@ -862,7 +999,7 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
 			/***** SAVE RECORDS *****/
 			Ext.each(editEls,function(editEl) {
 				var fieldname = this.get_fieldname_by_editEl(editEl);
-				if(!fieldname) { return; }
+				if(!this.fieldRecordCanEdit(fieldname,Record)) { return; }
 				
 				if(save) {
 					if(!this.save_field_data(editEl,fieldname,index,Record)) { 
@@ -954,7 +1091,6 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
     if(this.scrollNodeClass) {
       var nodeEl = new Ext.Element(node);
       if(nodeEl.hasClass(this.scrollNodeClass)) {
-        console.dir(node);
         return node;
       }
     }
@@ -970,3 +1106,67 @@ Ext.ux.RapidApp.AppDV.DataView = Ext.extend(Ext.DataView, {
   }
 });
 Ext.reg('appdv', Ext.ux.RapidApp.AppDV.DataView);
+
+Ext.ux.RapidApp.AppDV.FieldPlugin = Ext.extend(Ext.util.Observable,{
+
+  constructor: function(config){
+    this.addEvents('valid','invalid');
+    this.on('valid',this.onFieldValid,this);
+    this.on('invalid',this.onFieldInvalid,this);
+  },
+    
+  init: function(Field) {
+    var Ctx = Field.AppDv_context;
+    if(!Ctx) { return; }
+    
+    this.Field = Field; 
+    this.Ctx = Ctx;
+    
+    var recNode = Ctx.appdv.getNode(Ctx.index);
+    if(recNode) { 
+      Ctx.recEl = new Ext.Element(recNode); 
+    }
+
+    this.relayEvents(Field,['valid','invalid']);
+  },
+    
+  onFieldValid: function() {
+    var El = this.Ctx.editEl;
+    if(!El) { return; }
+    
+    if(El.hasClass('appdv-field-invalid')) {
+      El.removeClass('appdv-field-invalid');
+    }
+    this.checkRecEl.defer(100,this);
+  },
+  onFieldInvalid: function(Field,msg) {
+    var El = this.Ctx.editEl, recEl = this.Ctx.recEl;
+    if(!El) { return; }
+
+    if(!El.hasClass('appdv-field-invalid')) {
+      El.addClass('appdv-field-invalid');
+    }
+    
+    if(recEl && !recEl.hasClass('appdv-rec-invalids')) {
+      recEl.addClass('appdv-rec-invalids');
+    }
+  },
+  checkRecEl: function() {
+    var recEl = this.Ctx.recEl;
+
+    if(! recEl || ! recEl.hasClass('appdv-rec-invalids')) {
+      return;
+    }
+    
+    // If the record is marked invalid, but there are no longer
+    // any field invalids, clear it
+    if(! recEl.child('div.appdv-field-invalid')){
+      if(recEl.hasClass('appdv-rec-invalids')) {
+        recEl.removeClass('appdv-rec-invalids');
+      }
+    }
+  }
+   
+
+});
+Ext.preg('appdv-field-plugin',Ext.ux.RapidApp.AppDV.FieldPlugin);
